#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
This module implements community detection.
"""
from __future__ import print_function
__all__ = ["partition_at_level", "modularity", "best_partition", "generate_dendrogram", "generate_dendogram", "induced_graph"]
__author__ = """Thomas Aynaud (thomas.aynaud@lip6.fr)"""
#    Copyright (C) 2009 by
#    Thomas Aynaud <thomas.aynaud@lip6.fr>
#    All rights reserved.
#    BSD license.

__PASS_MAX = -1
__MIN = 0.0000001

import networkx as nx
import sys
import types
import array
import copy
import operator
import collections as col
import sklearn.metrics.cluster as sk


def partition_at_level(dendrogram, level) :
    """Return the partition of the nodes at the given level

    A dendrogram is a tree and each level is a partition of the graph nodes.
    Level 0 is the first partition, which contains the smallest communities, and the best is len(dendrogram) - 1.
    The higher the level is, the bigger are the communities
    This code remains unchanged for multilayer

    Parameters
    ----------
    dendrogram : list of dict
       a list of partitions, ie dictionnaries where keys of the i+1 are the values of the i.
    level : int
       the level which belongs to [0..len(dendrogram)-1]

    Returns
    -------
    partition : dictionnary
       A dictionary where keys are the nodes and the values are the set it belongs to

    Raises
    ------
    KeyError
       If the dendrogram is not well formed or the level is too high

    See Also
    --------
    best_partition which directly combines partition_at_level and generate_dendrogram to obtain the partition of highest modularity

    Examples
    --------
    >>> G=nx.erdos_renyi_graph(100, 0.01)
    >>> dendo = generate_dendrogram(G)
    >>> for level in range(len(dendo) - 1) :
    >>>     print "partition at level", level, "is", partition_at_level(dendo, level)
    """
    partition = dendrogram[0].copy()
    for index in range(1, level + 1) :
        for node, community in partition.items() :
            partition[node] = dendrogram[index][community]
    return partition


def modularity(partition, graph) :
    """Compute the modularity of a partition of a graph
    Updated for multilayer case
    Parameters
    ----------
    partition : dict
       the partition of the nodes, i.e a dictionary where keys are their nodes and values the communities
    graph : A list of networkx.Graphs
       the networkx graph which is decomposed

    Returns
    -------
    modularity : float
       The modularity

    Raises
    ------
    KeyError
       If the partition is not a partition of all graph nodes
    ValueError
        If the graph has no link
    TypeError
        If graph is not a networkx.Graph

    References
    ----------
    .. 1. Newman, M.E.J. & Girvan, M. Finding and evaluating community structure in networks. Physical Review E 69, 26113(2004).

    Examples
    --------
    >>> G=nx.erdos_renyi_graph(100, 0.01)
    >>> part = best_partition(G)
    >>> modularity(part, G)
    """
    for x in graph:
        if type(x) != nx.Graph :
            raise TypeError("Bad graph type, use only non directed graph")
    incfinal=[]
    degfinal=[]
    links=[]
    for x in graph:
        links.append(float(x.size()))
        inc = dict([])
        deg = dict([])
        for node in x.nodes():
            com = partition[node]
            deg[com] = deg.get(com, 0.) + x.degree(node, weight = 'weight')
            for neighbor, datas in x[node].items() :
                weight = datas.get("weight", 1)
                if partition[neighbor] == com :
                   if neighbor == node :
                      inc[com] = inc.get(com, 0.) + float(weight)
                   else :
                      inc[com] = inc.get(com, 0.) + float(weight) / 2.
        degfinal.append(deg)
        incfinal.append(inc)
    M=len(graph)
    res = 0.
    for m in range(M) :
        for com in set(partition.values()) :
            res += (links[m]/sum(links))*((incfinal[m].get(com, 0.) / links[m]) - (degfinal[m].get(com, 0.) / (2.*links[m]))**2)
    return res


def best_partition(graph, partition = None) :
    """Compute the partition of the graph nodes which maximises the modularity
    (or try..) using the Louvain heuristices

    This is the partition of highest modularity, i.e. the highest partition of the dendrogram
    generated by the Louvain algorithm.

    Parameters
    ----------
    graph : networkx.Graph
       the networkx graph which is decomposed
    partition : dict, optionnal
       the algorithm will start using this partition of the nodes. It's a dictionary where keys are their nodes and values the communities

    Returns
    -------
    partition : dictionnary
       The partition, with communities numbered from 0 to number of communities

    Raises
    ------
    NetworkXError
       If the graph is not Eulerian.

    See Also
    --------
    generate_dendrogram to obtain all the decompositions levels

    Notes
    -----
    Uses Louvain algorithm

    References
    ----------
    .. 1. Blondel, V.D. et al. Fast unfolding of communities in large networks. J. Stat. Mech 10008, 1-12(2008).

    Examples
    --------
    >>>  #Basic usage
    >>> G=nx.erdos_renyi_graph(100, 0.01)
    >>> part = best_partition(G)

    >>> #other example to display a graph with its community :
    >>> #better with karate_graph() as defined in networkx examples
    >>> #erdos renyi don't have true community structure
    >>> G = nx.erdos_renyi_graph(30, 0.05)
    >>> #first compute the best partition
    >>> partition = community.best_partition(G)
    >>>  #drawing
    >>> size = float(len(set(partition.values())))
    >>> pos = nx.spring_layout(G)
    >>> count = 0.
    >>> for com in set(partition.values()) :
    >>>     count = count + 1.
    >>>     list_nodes = [nodes for nodes in partition.keys()
    >>>                                 if partition[nodes] == com]
    >>>     nx.draw_networkx_nodes(G, pos, list_nodes, node_size = 20,
                                    node_color = str(count / size))
    >>> nx.draw_networkx_edges(G,pos, alpha=0.5)
    >>> plt.show()
    """
    dendo = generate_dendrogram(graph, partition)
    return partition_at_level(dendo, len(dendo) - 1 )


def generate_dendogram(graph, part_init = None) :
    """Deprecated, use generate_dendrogram"""
    return generate_dendrogram(graph, part_init)
    
    
def generate_dendrogram(graph, part_init = None) :
    """Find communities in the graph and return the associated dendrogram

    A dendrogram is a tree and each level is a partition of the graph nodes.  Level 0 is the first partition, which contains the smallest communities, and the best is len(dendrogram) - 1. The higher the level is, the bigger are the communities


    Parameters
    ----------
    graph : networkx.Graph
        the networkx graph which will be decomposed
    part_init : dict, optionnal
        the algorithm will start using this partition of the nodes. It's a dictionary where keys are their nodes and values the communities

    Returns
    -------
    dendrogram : list of dictionaries
        a list of partitions, ie dictionnaries where keys of the i+1 are the values of the i. and where keys of the first are the nodes of graph

    Raises
    ------
    TypeError
        If the graph is not a networkx.Graph

    See Also
    --------
    best_partition

    Notes
    -----
    Uses Louvain algorithm

    References
    ----------
    .. 1. Blondel, V.D. et al. Fast unfolding of communities in large networks. J. Stat. Mech 10008, 1-12(2008).

    Examples
    --------
    >>> G=nx.erdos_renyi_graph(100, 0.01)
    >>> dendo = generate_dendrogram(G)
    >>> for level in range(len(dendo) - 1) :
    >>>     print "partition at level", level, "is", partition_at_level(dendo, level)
    """
    for x in graph:
        if type(x) != nx.Graph :
            raise TypeError("Bad graph type, use only non directed graph")
        if x.number_of_edges() == 0 :
            part = dict([])
            for node in x.nodes() :
                part[node] = node
            return part

    M=len(graph)
    current_graph = copy.deepcopy(graph)
    status = Status()
    status.init(current_graph, part_init)
    mod = __modularity(status)
    status_list = list()
    __one_level(current_graph, status)
    new_mod = __modularity(status)
    partition = __renumber(status.node2com)
    status_list.append(partition)
    mod = new_mod
    current_graph = induced_graph(partition, current_graph)
    status.init(current_graph)

    while True :
        __one_level(current_graph, status)
        new_mod = __modularity(status)
        if new_mod - mod < __MIN :
            break
        partition = __renumber(status.node2com)
        status_list.append(partition)
        mod = new_mod
        current_graph = induced_graph(partition, current_graph)
        status.init(current_graph)
    return status_list[:]


def induced_graph(partition, graph) :
    """Changes made for multilayer : graph is a list of graphs.
    Produce the graph where nodes are the communities

    there is a link of weight w between communities if the sum of the weights of the links between their elements is w

    Parameters
    ----------
    partition : dict
       a dictionary where keys are graph nodes and  values the part the node belongs to
    graph : networkx.Graph
        the initial graph

    Returns
    -------
    g : networkx.Graph
       a networkx graph where nodes are the parts

    Examples
    --------
    >>> n = 5
    >>> g = nx.complete_graph(2*n)
    >>> part = dict([])
    >>> for node in g.nodes() :
    >>>     part[node] = node % 2
    >>> ind = induced_graph(part, g)
    >>> goal = nx.Graph()
    >>> goal.add_weighted_edges_from([(0,1,n*n),(0,0,n*(n-1)/2), (1, 1, n*(n-1)/2)])
    >>> nx.is_isomorphic(int, goal)
    True
    """
    ret=[]
    M=len(graph)
    for m in range(M):
        ret.append(nx.Graph())
        ret[m].add_nodes_from(partition.values())
        for node1, node2, datas in graph[m].edges_iter(data = True) :
            weight = datas.get("weight", 1)
            com1 = partition[node1]
            com2 = partition[node2]
            w_prec = ret[m].get_edge_data(com1, com2, {"weight":0}).get("weight", 1)
            ret[m].add_edge(com1, com2, weight = w_prec + weight)
    return ret


def __renumber(dictionary) :
    """Renumber the values of the dictionary from 0 to n
       possibly unchanged.
    """
    count = 0
    ret = dictionary.copy()
    new_values = dict([])

    for key in dictionary.keys() :
        value = dictionary[key]
        new_value = new_values.get(value, -1)
        if new_value == -1 :
            new_values[value] = count
            new_value = count
            count = count + 1
        ret[key] = new_value

    return ret


def __load_binary(data) :
    """Load binary graph as used by the cpp implementation of this algorithm
    """
    data = open(data, "rb")

    reader = array.array("I")
    reader.fromfile(data, 1)
    num_nodes = reader.pop()
    reader = array.array("I")
    reader.fromfile(data, num_nodes)
    cum_deg = reader.tolist()
    num_links = reader.pop()
    reader = array.array("I")
    reader.fromfile(data, num_links)
    links = reader.tolist()
    graph = nx.Graph()
    graph.add_nodes_from(range(num_nodes))
    prec_deg = 0

    for index in range(num_nodes) :
        last_deg = cum_deg[index]
        neighbors = links[prec_deg:last_deg]
        graph.add_edges_from([(index, int(neigh)) for neigh in neighbors])
        prec_deg = last_deg

    return graph


def __one_level(graph, status) :
    """Compute one level of communities

    """
    modif = True
    nb_pass_done = 0
    cur_mod = __modularity(status)
    new_mod = cur_mod
    M=len(graph)
    while modif  and nb_pass_done != __PASS_MAX :
        cur_mod = new_mod
        modif = False
        nb_pass_done += 1
        for node in graph[0].nodes() :
            neigh_weight1=[]
            neigh_weight2=[]
            com_node = status.node2com[node]
            neigh_communities = __neighcom(node, graph, status)
            for m in range(M):
                neigh_weight1.append(neigh_communities[m].get(com_node,0.))
            __remove(node, com_node,
                    neigh_weight1, status)
            best_increase = 0.
            incr={}
            incr[com_node]=0.
            for m in range(M):
                for com in list(neigh_communities[m]) :
                    degc_totw=status.gdegrees[m].get(node, 0.) / (status.total_weight[m]*2.)
                    incr[com]=incr.get(com,0.)+ neigh_communities[m][com]/sum(status.total_weight)  - (status.degrees[m].get(com, 0.) * degc_totw/sum(status.total_weight))               
            best_com=max(incr.items(), key=operator.itemgetter(1))[0]
            for m in range(M):
                neigh_weight2.append(neigh_communities[m].get(best_com,0.))

            __insert(node, best_com,
                    neigh_weight2, status)
            if best_com != com_node :
                modif = True
        new_mod = __modularity(status)
        if new_mod - cur_mod < __MIN :
            break


class Status :
    """
    To handle several data in one struct.
    Modifications made for multilayer graphs
    Could be replaced by named tuple, but don't want to depend on python 2.6
    """
    node2com = {}
    total_weight =[]
    internals = []
    degrees = []
    gdegrees = []
    loops=[]

    def __init__(self) :
        self.node2com = dict([])
        self.total_weight = []
        self.degrees = list()
        self.gdegrees = list()
        self.internals = list()
        self.loops = list()

    def __str__(self) :
        return ("node2com : " + str(self.node2com) + " degrees : "
            + str(self.degrees) + " internals : " + str(self.internals)
            + " total_weight : " + str(self.total_weight))

    def mcopy(self) :
        """Perform a deep copy of status"""
        new_status = Status()
        new_status.node2com = copy.deepcopy(self.node2com)
        new_status.internals = copy.deepcopy(self.internals)
        new_status.degrees = copy.deepcopy(self.degrees)
        new_status.gdegrees = copy.deepcopy(self.gdegrees)
        new_status.total_weight = copy.deepcopy(self.total_weight)
        new_status.loops = copy.deepcopy(self.loops)

    def init(self, graph, part = None) :
        """Initialize the status of a graph with every node in one community"""
        self.node2com = dict([])
        self.total_weight = []
        self.degrees = list()
        self.gdegrees = list()
        self.internals = list()
        self.loops=list()
        for x in graph:
            self.total_weight.append(x.size(weight='weight'))
        
        if part == None :
            for x in graph:
                tempdegrees={}
                tempgdegrees={}
                temploops={}
                tempinternals={}
                count=0
                for node in x.nodes() :
                    self.node2com[node] = count
                    deg = float(x.degree(node, weight = 'weight'))
                    if deg < 0 :
                        raise ValueError("Bad graph type, use positive weights")
                    tempdegrees[count] = deg
                    tempgdegrees[node] = deg
                    temploops[node] = float(x.get_edge_data(node, node,
                                                 {"weight":0}).get("weight", 1))
                    tempinternals[count] = temploops[node]
                    count = count + 1

                self.degrees.append(tempdegrees)
                self.gdegrees.append(tempgdegrees)
                self.loops.append(temploops)
                self.internals.append(tempinternals)

        else :
            for x in graph:
                tempdegrees={}
                tempgdegrees={}
                temploops={}
                tempinternals={}
                for node in x.nodes() :
                    com = part[node]
                    self.node2com[node] = com
                    deg = float(x.degree(node, weight = 'weight'))
                    tempdegrees[com] = tempdegrees.get(com, 0) + deg
                    tempgdegrees[node] = deg
                    inc = 0.
                    for neighbor, datas in x[node].items() :
                        weight = datas.get("weight", 1)
                        if weight <= 0 :
                            raise ValueError("Bad graph type, use positive weights")
                        if part[neighbor] == com :
                            if neighbor == node :
                                inc += float(weight)
                            else :
                                inc += float(weight) / 2.
                    tempinternals[com] = tempinternals.get(com, 0) + inc

                self.degrees.append(tempdegrees)
                self.gdegrees.append(tempgdegrees)
                self.loops.append(temploops)
                self.internals.append(tempinternals)



def __neighcom(node, graph, status) :
    """
    Compute the communities in the neighborood of node in the graph given
    with the decomposition node2com
    """
    weights = list()
    for x in graph:
        tempweights={}
        for neighbor, datas in x[node].items() :
            if neighbor != node :
                weight = datas.get("weight", 1)
                neighborcom = status.node2com[neighbor]
                tempweights[neighborcom] = tempweights.get(neighborcom, 0) + weight
        weights.append(tempweights)
    return weights


def __remove(node, com, weight, status) :
    """ Remove node from community com and modify status"""
    M=len(weight)
    for m in range(M):
        status.degrees[m][com] = ( status.degrees[m].get(com, 0.)
                                    - status.gdegrees[m].get(node, 0.) )
        status.internals[m][com] = float( status.internals[m].get(com, 0.) -
                weight[m] - status.loops[m].get(node, 0.) )
    status.node2com[node] = -1


def __insert(node, com, weight, status) :
    """ Insert node into community and modify status"""
    status.node2com[node] = com
    M=len(weight)
    for m in range(M):
        status.degrees[m][com] = ( status.degrees[m].get(com, 0.) +
                                status.gdegrees[m].get(node, 0.) )
        status.internals[m][com] = float( status.internals[m].get(com, 0.) +
                        weight[m] + status.loops[m].get(node, 0.) )


def __modularity(status) :
    """
    Compute the modularity of the partition of the graph faslty using status precomputed
    """
    M=len(status.total_weight)
    links=[]
    for m in range(M):
        links.append(float(status.total_weight[m]))
    result = 0.
    for m in range(M):
        for community in set(status.node2com.values()) :
            in_degree = status.internals[m].get(community, 0.)
            degree = status.degrees[m].get(community, 0.)
            if links[m] > 0 :
                result = result + (links[m]/sum(links))*(in_degree / links[m] - ((degree / (2.*links[m]))**2))
    return result


def calculate_purity(truepart, mypart):
    """
        Calculate the purity, a measurement of quality for the clustering 
        results.
        
        Each cluster is assigned to the class which is most frequent in the 
        cluster.  Using these classes, the percent accuracy is then calculated.
        
        Returns:
          A number between 0 and 1.  Poor clusterings have a purity close to 0 
          while a perfect clustering has a purity of 1.
          
        Raises:
          UnlabelledDataSetError if the dataset is not labelled.
    """
    # get the set of unique cluster ids
    clusters = set(mypart.values())

    # find out what class is most frequent in each cluster
    cluster_classes = {}
    for cluster in clusters:
        # get the indices of rows in this cluster
        indices=[]
        for node in mypart:
            if mypart[node]== cluster:
                indices.append(truepart[node])
        # assign the most common label to be the label for this cluster
        cluster_classes[cluster] = max(set(indices), key=indices.count)
        
        
    # get the list of labels as determined by each cluster's most frequent 
    # label
    labels_mypart={}
    # See how the clustering labels compare with the actual labels.  
    # Return the percentage of indices in agreement.
    num_agreed = 0
    for node in mypart:
        labels_mypart[node]=cluster_classes[mypart[node]]
        if labels_mypart[node] == truepart[node]:
            num_agreed += 1
        
    return float(num_agreed) / len(mypart)
        
def calculate_nmi(truepart, mypart):
    labels_true=[]
    labels_pred=[]
    for node in mypart:
        labels_true.append(truepart[node])
        labels_pred.append(mypart[node])
    nmi=sk.normalized_mutual_info_score(labels_true, labels_pred)
    return nmi

        

def main() :
    """Main function to mimic C++ version behavior"""
    try :
        filename = sys.argv[1]
        graphfile = __load_binary(filename)
        partition = best_partition(graphfile)
        print(str(modularity(partition, graphfile)), file=sys.stderr)
        for elem, part in partition.items() :
            print(str(elem) + " " + str(part))
    except (IndexError, IOError):
        print("Usage : ./community filename")
        print("find the communities in graph filename and display the dendrogram")
        print("Parameters:")
        print("filename is a binary file as generated by the ")
        print("convert utility distributed with the C implementation")
